/*
 * Copyright (c) 2001 Conexant Systems, Inc.
 * 
 * 1.   Permitted use. Redistribution and use in source and binary forms,
 * with or without modification, are permitted under the terms set forth
 * herein.
 * 
 * 2.   Disclaimer of Warranties. CONEXANT AND OTHER CONTRIBUTORS MAKE NO
 * REPRESENTATION ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.
 * IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTIES OF ANY KIND.
 * CONEXANT AND OTHER CONTRIBUTORS DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, GOOD TITLE AND AGAINST INFRINGEMENT.
 * 
 * This software has not been formally tested, and there is no guarantee that
 * it is free of errors including, but not limited to, bugs, defects,
 * interrupted operation, or unexpected results. Any use of this software is
 * at user's own risk.
 * 
 * 3.   No Liability.
 * 
 * (a) Conexant or contributors shall not be responsible for any loss or
 * damage to Company, its customers, or any third parties for any reason
 * whatsoever, and CONEXANT OR CONTRIBUTORS SHALL NOT BE LIABLE FOR ANY
 * ACTUAL, DIRECT, INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL, OR CONSEQUENTIAL
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED, WHETHER IN CONTRACT, STRICT OR OTHER LEGAL THEORY OF
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * (b) User agrees to hold Conexant and contributors harmless from any
 * liability, loss, cost, damage or expense, including attorney's fees,
 * as a result of any claims which may be made by any person, including
 * but not limited to User, its agents and employees, its customers, or
 * any third parties that arise out of or result from the manufacture,
 * delivery, actual or alleged ownership, performance, use, operation
 * or possession of the software furnished hereunder, whether such claims
 * are based on negligence, breach of contract, absolute liability or any
 * other legal theory.
 * 
 * 4.   Notices. User hereby agrees not to remove, alter or destroy any
 * copyright, trademark, credits, other proprietary notices or confidential
 * legends placed upon, contained within or associated with the Software,
 * and shall include all such unaltered copyright, trademark, credits,
 * other proprietary notices or confidential legends on or in every copy of
 * the Software.
 * 
 */
/*****************************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>

#include "framewrk.h"
#include "ossysenv.h"
/*****************************************************************************/
#define MAX_LINE_LEN         256

/*****************************************************************************/
inline unsigned char ConvertHexDigitToNum(const char HexDigit);

/*****************************************************************************/

int main(int argc, char **argv)
{
	FILE            *fpHex = NULL;               // hex source file
	FILE            *fpBin = NULL;               // target binary file
	char            HexNo[3];                    // temp var to symbol 2 charcters as a char no.
	char            HexLine[MAX_LINE_LEN+1];     // hex text line
	UINT32          BinDataSize = 0;             // (size of the output binary file) - (the first ulong)
	unsigned char   TargetLine[MAX_LINE_LEN];    // binary line for write to target
	unsigned char   NumCount = 0;				 // number of Hex bumbers in the line
	unsigned char   Num,i;

	
	if (3 != argc)
	{
		printf("usage : text2bin <FIRMWARE_FILENAME> <OUTPUTFILE>\n");
		return 1;
	}

	/* open the file to read */
	fpHex = fopen(argv[1],"r");
	if (NULL == fpHex)
	{
		printf("Error : can't open Yukon source file '%s' .\n",argv[1]);
		return 2;
	}
	/* open the target binary file */
	fpBin = fopen(argv[2],"wb");
	if (NULL == fpBin)
	{
		printf("Error : can't open Yukon target file '%s' .\n",argv[2]);
		fclose (fpHex);
		return 3;
	}

		

	/* loop on all the lines - creating the binary file */
	fseek(fpBin, sizeof(BinDataSize), SEEK_SET); 	/* save the first bytes to file size and lines number */

	BinDataSize  = 0;

	while ( NULL != fgets(HexLine, MAX_LINE_LEN, fpHex))
	{
		if (HexLine[0] != ':')
		{
			printf("Error : Bad target file '%s' - line doesn't start with ':' .\n",argv[1]);
			fclose(fpHex);
			fclose(fpBin);
			return 4;
		}

		NumCount = 	(strlen(HexLine) - 1)/2;
		BinDataSize += NumCount;

		// build the line to write to bin file
		for (i=0 ; i<NumCount ; i++)
		{	
			Num = 0;
			strncpy(HexNo,HexLine + (2*i) + 1,2);
			HexNo[2] = '\0';
			Num = (16 * ConvertHexDigitToNum(HexNo[0])) + (ConvertHexDigitToNum(HexNo[1]));
			TargetLine[i] = Num;
		}

		// write the line to bin file 
		fwrite(TargetLine,sizeof(unsigned char),NumCount,fpBin);
	}

	fseek(fpBin,0,SEEK_SET);
	fwrite((PVOID)&BinDataSize, sizeof(BinDataSize), 1, fpBin);
	
	/* close Yukon file */
	fclose( fpHex );
	fclose( fpBin );

	return 0;
}

/*****************************************************************************/
inline unsigned char ConvertHexDigitToNum( const char HexDigit )
{
	unsigned char temp = 0;

	if ( isdigit( HexDigit ) )
	{
		temp = (HexDigit - '0');
	}
	else if (HexDigit <= 'f' && HexDigit >= 'a')
	{
		temp = ((HexDigit - 'a')+10);
	}
	else if (HexDigit <= 'F' && HexDigit >= 'A')
	{
		temp = ( (HexDigit - 'A') + 10 );
	}
	return ( temp );
}
