/*
 * Copyright (c) 2001 Conexant Systems, Inc.
 * 
 * 1.   Permitted use. Redistribution and use in source and binary forms,
 * with or without modification, are permitted under the terms set forth
 * herein.
 * 
 * 2.   Disclaimer of Warranties. CONEXANT AND OTHER CONTRIBUTORS MAKE NO
 * REPRESENTATION ABOUT THE SUITABILITY OF THIS SOFTWARE FOR ANY PURPOSE.
 * IT IS PROVIDED "AS IS" WITHOUT EXPRESS OR IMPLIED WARRANTIES OF ANY KIND.
 * CONEXANT AND OTHER CONTRIBUTORS DISCLAIMS ALL WARRANTIES WITH REGARD TO
 * THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A PARTICULAR PURPOSE, GOOD TITLE AND AGAINST INFRINGEMENT.
 * 
 * This software has not been formally tested, and there is no guarantee that
 * it is free of errors including, but not limited to, bugs, defects,
 * interrupted operation, or unexpected results. Any use of this software is
 * at user's own risk.
 * 
 * 3.   No Liability.
 * 
 * (a) Conexant or contributors shall not be responsible for any loss or
 * damage to Company, its customers, or any third parties for any reason
 * whatsoever, and CONEXANT OR CONTRIBUTORS SHALL NOT BE LIABLE FOR ANY
 * ACTUAL, DIRECT, INDIRECT, SPECIAL, PUNITIVE, INCIDENTAL, OR CONSEQUENTIAL
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED, WHETHER IN CONTRACT, STRICT OR OTHER LEGAL THEORY OF
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY
 * WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 * 
 * (b) User agrees to hold Conexant and contributors harmless from any
 * liability, loss, cost, damage or expense, including attorney's fees,
 * as a result of any claims which may be made by any person, including
 * but not limited to User, its agents and employees, its customers, or
 * any third parties that arise out of or result from the manufacture,
 * delivery, actual or alleged ownership, performance, use, operation
 * or possession of the software furnished hereunder, whether such claims
 * are based on negligence, breach of contract, absolute liability or any
 * other legal theory.
 * 
 * 4.   Notices. User hereby agrees not to remove, alter or destroy any
 * copyright, trademark, credits, other proprietary notices or confidential
 * legends placed upon, contained within or associated with the Software,
 * and shall include all such unaltered copyright, trademark, credits,
 * other proprietary notices or confidential legends on or in every copy of
 * the Software.
 * 
 */
#include "oscompat.h"

#include "framewrk.h"
#include "ossysenv.h"

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
#include <asm/i387.h>
#endif

#if defined(cpu_has_xmm) && !defined(load_mxcsr)
#define load_mxcsr( val ) do { \
	unsigned long __mxcsr = ((unsigned long)(val) & 0xffbf); \
	asm volatile( "ldmxcsr %0" : : "m" (__mxcsr) ); \
} while (0)
#endif

#define NFPSTATES 10

typedef struct {
	unsigned long used;
	union i387_union i387;
	unsigned int cr0;
} fpstate_t;

static fpstate_t fpstates[NFPSTATES];

static inline void
hsf_kernel_fpu_begin(fpstate_t *fpstate)
{
	// XXX if CONFIG_PREEMPT we might have to disable preemption here
	fpstate->cr0 = read_cr0();
	clts();
#ifdef cpu_has_fxsr
	if ( cpu_has_fxsr ) {
		asm volatile( "fxsave %0 ; fnclex"
		      	: "=m" (fpstate->i387.fxsave) );
	} else {
		asm volatile( "fnsave %0 ; fwait"
		      	: "=m" (fpstate->i387.fsave) );
	}
#else
	asm volatile( "fnsave %0 ; fwait"
	      	: "=m" (fpstate->i387) );
#endif

    __asm__("fninit");
#ifdef cpu_has_xmm
	if ( cpu_has_xmm )
		load_mxcsr(0x1f80);
#endif
}

static inline void
hsf_kernel_fpu_end(fpstate_t *fpstate)
{
	clts();
#ifdef cpu_has_fxsr
	if ( cpu_has_fxsr ) {
		asm volatile( "fxrstor %0; fwait"
			      : : "m" (fpstate->i387.fxsave) );
	} else {
		asm volatile( "frstor %0; fwait"
			      : : "m" (fpstate->i387.fsave) );
	}
#else
	asm volatile( "frstor %0; fwait"
		      : : "m" (fpstate->i387) );
#endif
	write_cr0(fpstate->cr0);
}

/***************************************************************************
Function name :	OSFloatPrefix
Parameters :	None
Return value :	None
functionality :	Treats the FPU in order to enable float operations.
***************************************************************************/
 
GLOBAL	int OSFloatPrefix(void)
{
	static int Id;

	ASSERT(in_interrupt());

	while(test_and_set_bit(0, &fpstates[Id].used)) {
		Id++;
		if(Id == NFPSTATES)
			Id = 0;
	}

	hsf_kernel_fpu_begin(&fpstates[Id]);

	/* not allowed to sleep until OSFloatSuffix().
	 * moot point, since we're in interrupt context anyway..
	 * */
	return Id;
}

/***************************************************************************
Function name :	OSFloatPrefix
Parameters :	None
Return value :	None
functionality :	Restores the FPU after float operations.
***************************************************************************/
GLOBAL	BOOL OSFloatSuffix(int Id)
{
	hsf_kernel_fpu_end(&fpstates[Id]);
	if(!test_and_clear_bit(0, &fpstates[Id].used)) {
		printk(KERN_ERR "%s: Id=%d used bit was FALSE !\n", __FUNCTION__, Id);
		return FALSE;
	}

	return TRUE;
}

